/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  YNOP (ynop@acm.org)
*   @version 1.0
*   @date    Feb 5 2000
*   @license Inferno SDK Code
*******************************************************/
#ifndef INFERNO_COMMON_H
#define INFERNO_COMMON_H

#ifdef __POWERPC__
#define BUILDING_I_CORE 1
#endif

#include <GraphicsDefs.h>
#include <Point.h>
#include <List.h>
#include <OS.h>
#include <Bitmap.h>
#include <Looper.h>

class InfernoLayer;
class InfernoCMS;
class Gradient;

// This is the Brush structure
// We suggest you use the defulat modes defined
// above. If you wish to make your own then you 
// should use MODE_COUNT + x so it does not 
// conflict with defaults
struct IBrush{
   BBitmap *brush;
   BPoint center;
   int32 mode;
   int32 stepping; // percentage of stepping (1 to 100) 
};

// things that can be used as the first parameter
// of SendMessage .. they have special meaning
#define INFERNO_CONTROL_MESSAGE 'Ipst'  
#define I_COMMON_CHANGED        'Icom'  // Common changed control msg
#define INFERNO_APP             "Inferno" // App target for SendMessage
#define BROADCAST               "Broadcast" // Broadcast to all tools/floaters and Infenro

#define I_COLOR_CHANGED   'IcmC'  // Message sent out on color changes
#define I_BRUSH_CHANGED   'IcmB'  // Message sent out when the bursh changes
#define I_PATTERN_CHANGED 'IcmP'  // Message sent out when the pattern changes
#define I_GRADIENT_CHANGED 'IcmG' // Message sent out when the gradient changes

// There are a few CMS entrys that the core installs
// The Display cms is used for drawing to the canvase
// every canvas uses the same CMS.  The Printer CMS is
// used when passing data to the default beos printer.
// If you wish to use these see the RequestCMS() method
#define DISPLAY_CMS        "Display" // Default CMS requester for Screen Display
#define PRINTER_CMS        "Printer" // Default CMS requester for internal Printing

class InfernoCommon{
public:
   InfernoCommon();
   ~InfernoCommon();
    
   // This is the driver for updateing the view
   // the tool is responcible for calling this to
   // let inferno know that it has changed the image.
   void UpdateCanvas(BRect dirtyRect);
   void UpdateCanvas();
    
   // Methods for accessing current color settings
   // Note that Setting the color will broadcast a 
   // message to all the tools
   rgb_color GetFgColor() const;
   rgb_color GetBgColor() const;
   void SetFgColor(rgb_color c);
   void SetBgColor(rgb_color c);
   void SetFgColor(uint8 r,uint8 g, uint8 b, uint8 a);
   void SetBgColor(uint8 r,uint8 g, uint8 b, uint8 a);
   
   // Zoom factor methods
   // 200% is double size
   // 100% is normal zoom
   // 0% is no image :P
   void SetZoom(int32 precent = 100);
   int32 GetZoom() const;
   
   // Methods for getting and setting
   // the Brush ...
   IBrush* GetBrush() const;
   void SetBrush(IBrush *b);
   
   // Stuff uses for fill patterns
   BBitmap* GetPattern() const;
   void SetPattern(BBitmap *pat);
   
   // Get set gradiant...
   Gradient* GetGradient() const;
   void SetGradient(Gradient *g);
      
   // Things to let your plugin give the user 
   // a little feadback to how things are going
   // UpdateProgress takes a precentage from 0-100
   void SetStatus(const char *format,...) const;
   void UpdateProgress(int32 precent) const;
   
   // this is what enables us to use Layer Plugins
   // the one you may be interested in is the 
   // CreateLayerOfType("Bitmap",NULL);
   InfernoLayer* CreateLayerOfType(const char *type,BMessage* initmsg) const;
   // Returns a list of char* to the names off all the currently
   // installed Layer Types.
   BList* GetInstalledLayerTypes() const;
   
   // Adds a layer to the current project. 
   // If you pass -1 in it adds at the current position
   status_t AddLayer(InfernoLayer *ILay, int32 i = -1);
   
   // Send a message to any Tool, Addon, or Inferno app
   // this could also send messages to things that register
   // themselves as Inferno helpers (well talk about that later)
   void SendMessage(const char *name,BMessage *msg);
      
   // Get a language specific key from the users language file
   // To install a new key append your pluginname_key to the 
   // appropriate language file
   const char* GetLanguageKey(const char *key) const;
   
   // Log some data to the Inferno log
   void Log(const char *format,...) const;
      
   // Add a tool top to a view
   void AddTip(BView *v, const char *tip = NULL);
   
   // Installs a handler for a drop on the canvas.
   // A plain/text drop would most likely be handled
   // by the TextLayer (InstallDropHandler("plain/text",this,"Create TextLayer"))
   // More than one target can exist for each mime. The core
   // will let the user decide at that point.
   void InstallDropHandler(const char *mime ,BLooper *target, const char *name);
   
   // Requests a CMS by a particular name. If that name does not
   // exist then the core will add that name to the avalable list
   // in the user prefs panel and will prompt the user if they would
   // like to set up the new CMS.  It will return the newly set up
   // CMS or NULL if non is selected. If the CMS is already installed
   // it will return that or NULL if the user does not want to use CMS
   // in this case.
   void InstallCMSReqester(const char *name);
   status_t CMSTransform(const char *name,BBitmap *bmap, BRect clip);
   status_t CMSTransform(const char *name, rgb_color col, rgb_color *newcol);
   
public:
   // These three vars should not be used in tools to calculate
   // or respond to user input. They are here so floater and other
   // things can get them to display to the user. They could be 
   // inacurate our out of date.
   BPoint MouseDownAt;      // Last mouse down location
   BPoint MouseCurrentlyAt; // Current mouse position
   bool Dragging;           // true is mousedown & mousemove
private:
   void _ReservedCommon1();
   void _ReservedCommon2();
   void _ReservedCommon3();
   void _ReservedCommon4();

   InfernoCommon(const InfernoCommon&);
   InfernoCommon &operator=(const InfernoCommon&);
private:
   sem_id commonSem;
   rgb_color HighColor;
   rgb_color LowColor;
   IBrush *currentbrush;
   BBitmap *IPattern;
   Gradient *grad;
   uint32 _reserved[8];
};
#if BUILDING_I_CORE
extern InfernoCommon ICommon; // Included so you don't have to 
#endif
#endif





